<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 09/28/2008 *
 * ===================                          Last change: 09/28/2008 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : refback_functions.php                            *
 * -------------------------------------------------------------------- *
 * Short description : Functions for refback extension                  *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Funktionen fuer Refback-Erweiterung              *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2011 by Mailer Developer Team                   *
 * For more information visit: http://www.mxchange.org                  *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	die();
}

// Payback refback for refid and reduce it for current user
function addRefbackPoints ($userid, $ref, $points, $ref_points) {
	/* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'userid=' . $userid . ',ref=' . $ref . ',points=' . $points . ',ref_points=' . $ref_points . ' - ENTERED!');
	// Back ref depths
	$GLOBALS['refback_depth'][$userid][$ref] = $GLOBALS['ref_level'];

	// Init points
	$return = $ref_points;

	// Get all ref levels
	$result_refs = SQL_QUERY('SELECT `level`, `percents` FROM `{?_MYSQL_PREFIX?}_refdepths` WHERE `level` > 0 ORDER BY `level` ASC',
		__FUNCTION__, __LINE__);

	// "Walk" through all level
	while ($content = SQL_FETCHARRAY($result_refs)) {
		// Reset ref depths
		initReferalSystem();

		// "Walk" through all refids
		/* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'userid=' . $userid . ',ref=' . $ref . ',level=' . $content['level'] . ',percents=' . $content['percents'] . ',points=' . $points . ' - LOOP START!');
		foreach (getArrayFromRefbackLevel($userid, $content['level']) as $refid) {
			// Skip level zero or if both are the same
			if ($userid == $refid) {
				continue;
			} // END - if

			// Get refback percents
			$percents = getRefbackPercents($userid, $refid);
			/* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'userid=' . $userid . ',ref=' . $ref . ',refid=' . $refid . ',points=' . $points . ',percents=' . $percents);

			// Some percents given?
			if ($percents > 0) {
				/* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'points=' . $points . ',[percents]=' . $content['percents'] . ',percents=' . $percents . ',level=' . $content['level']);
				// Calculate points for refback
				$refback = $points * ($content['percents'] / 100) * ($percents / 100);

				// Update refback table ('refid' and 'userid' must be exchanged!)
				SQL_QUERY_ESC("UPDATE `{?_MYSQL_PREFIX?}_user_refs` SET `points`=`points`+%s WHERE `userid`=%s AND `refid`=%s LIMIT 1",
					array(
						$refback,
						$refid,
						$userid
					), __FUNCTION__, __LINE__);

				// Something must have been updated
				if (SQL_HASZEROAFFECTED()) {
					// Please report this bug
					debug_report_bug(__FUNCTION__, __LINE__, 'Nothing has been updated in user_refs,userid=' . $userid . ',refid=' . $refid . ',refback=' . $refback);
				} // END - if

				// Add points again, but only directly
				/* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'refback=' . $refback . ',userid=' . $userid . ',ref=' . $ref . ',refid=' . $refid . ' - UPDATE! (' . SQL_AFFECTEDROWS() . ')');
				addPointsThroughReferalSystem(sprintf("refback:%s", $refid), $userid, $refback);

				// Reduce points if refid is found
				if ($refid == $ref) {
					// Reduce points here!
					$return = $points * ($content['percents'] / 100) - $refback;
					/* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'return=' . $return . ' - REDUCED!');
				} // END - if
			} // END - if
		} // END foreach
	} // END - while

	// Free result
	SQL_FREERESULT($result_refs);

	// Restore ref depth
	$GLOBALS['ref_level'] = $GLOBALS['refback_depth'][$userid][$ref];

	// Return them
	/* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'userid=' . $userid . ',ref=' . $ref . ',points=' . $points . ',ref_points=' . $ref_points . ',return=' . $return . ' - EXIT!');
	return $return;
}

// "Getter" for refback percents
function getRefbackPercents ($userid, $ref) {
	/* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'userid=' . $userid . ',ref=' . $ref . ' - ENTERED!');
	// Skip identical ids
	if ($userid == $ref) {
		return 0;
	} // END - if

	// Is it cached?
	if (!isset($GLOBALS['refback_percents'][$userid][$ref])) {
		// Default is zero
		$GLOBALS['refback_percents'][$userid][$ref] = '0';

		// Get percents from database
		$result = SQL_QUERY_ESC("SELECT `refback` FROM `{?_MYSQL_PREFIX?}_user_refs` WHERE `userid`=%s AND `refid`=%s LIMIT 1",
			array($ref, $userid), __FUNCTION__, __LINE__);

		// Entry found? (Should be!)
		if (SQL_NUMROWS($result) == 1) {
			// Fetch percents
			list($GLOBALS['refback_percents'][$userid][$ref]) = SQL_FETCHROW($result);
			/* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'userid=' . $userid . ',ref=' . $ref . ',percents=' . $GLOBALS['refback_percents'][$userid][$ref]);
		} else {
			// Debug log
			logDebugMessage(__FUNCTION__, __LINE__, 'userid=' . $userid . ',ref=' . $ref . ' - No entry found. :-(');
		}

		// Free result
		SQL_FREERESULT($result);
	} // END - if

	// Return percents
	/* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'userid=' . $userid . ',ref=' . $ref . ',refback_percents=' . $GLOBALS['refback_percents'][$userid][$ref] . ' - EXIT!');
	return $GLOBALS['refback_percents'][$userid][$ref];
}

// "Getter" for userid array which will return only one entry
function getArrayFromRefbackLevel ($refid, $level) {
	/* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'refid=' . $refid . ',level=' . $level);
	// Init userids
	$userIds = array();

	// Look for all
	$result = SQL_QUERY_ESC("SELECT
	`userid`
FROM
	`{?_MYSQL_PREFIX?}_user_refs`
WHERE
	`refid`=%s AND `level`=%s
ORDER BY
	`userid` ASC",
		array($refid, $level), __FUNCTION__, __LINE__);

	// Entries found?
	/* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'numRows=' . SQL_NUMROWS($result));
	if (!SQL_HASZERONUMS($result)) {
		// Add all
		while ($content = SQL_FETCHARRAY($result)) {
			/* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'refid=' . $refid . ',level=' . $level . ',userid=' . $content['userid']);
			$userIds[] = $content['userid'];
		} // END - while
	} // END - if

	// Free result
	SQL_FREERESULT($result);

	// Return array
	/* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'refid=' . $refid . ',userIds()=' . count($userIds) . ' - EXIT!');
	return $userIds;
}

// Update "refback table"
function updateRefbackTable ($userid) {
	// Make it sure referal level zero (member him-/herself) is at least selected
	if (empty($GLOBALS['refback_level'])) {
		$GLOBALS['refback_level'] = NULL;
	} // END - if
	/* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'userid=' . $userid . ',level=' . $GLOBALS['refback_level'] . ' - ENTERED!');

	// Init refid and variables
	$GLOBALS['refback_refid'][$GLOBALS['refback_level']] = '0';
	$old = '0';
	$minus = '0';

	// Check for his referal
	$result_refid = SQL_QUERY_ESC("SELECT `userid` FROM `{?_MYSQL_PREFIX?}_user_data` WHERE `refid`=%s ORDER BY `userid` ASC",
		array(bigintval($userid)), __FUNCTION__, __LINE__);

	// When no entry was updated then we have to create it here
	/* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'SQL_NUMROWS()=' . SQL_NUMROWS($result_refid));
	if (!SQL_HASZERONUMS($result_refid)) {
		// Load all refids
		while (list($GLOBALS['refback_refid'][$GLOBALS['refback_level']]) = SQL_FETCHROW($result_refid)) {
			// Remmber userid
			$GLOBALS['refback_userid'][$GLOBALS['refback_refid'][$GLOBALS['refback_level']]] = $userid;

			// Refid set?
			/* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'userid=' . $userid . ',ref=' . $GLOBALS['refback_refid'][$GLOBALS['refback_level']]);
			if (($GLOBALS['refback_refid'][$GLOBALS['refback_level']] > 0) && ($GLOBALS['refback_refid'][$GLOBALS['refback_level']] != $userid) && (isset($GLOBALS['refback_refid'][$GLOBALS['refback_level']]))) {
				// New userid?
				if ((isset($GLOBALS['refback_refid'][$GLOBALS['refback_level']-1])) && (isset($GLOBALS['refback_userid'][$GLOBALS['refback_refid'][$GLOBALS['refback_level']-1]]))) {
					// New userid!
					$old = $userid;
					$userid = $GLOBALS['refback_userid'][$GLOBALS['refback_refid'][$GLOBALS['refback_level']-1]];
					$minus = 1;
					/* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'userid=' . $userid . ',old=' . $old . ',level=' . $GLOBALS['refback_level'] . ',ref=' . $GLOBALS['refback_refid'][$GLOBALS['refback_level']] . ' - NEW UID!');
				} // END - if

				// Check existence
				if (is_null($GLOBALS['refback_level'])) {
					// Level is NULL
					$result = SQL_QUERY_ESC("SELECT `id` FROM `{?_MYSQL_PREFIX?}_user_refs` WHERE `userid`=%s AND `level` IS NULL AND `refid`=%s LIMIT 1",
						array(
							bigintval($userid),
							bigintval($GLOBALS['refback_refid'][$GLOBALS['refback_level']])
						), __FUNCTION__, __LINE__);
				} else {
					// Level is > 0
					$result = SQL_QUERY_ESC("SELECT `id` FROM `{?_MYSQL_PREFIX?}_user_refs` WHERE `userid`=%s AND `level`=%s AND `refid`=%s LIMIT 1",
						array(
							bigintval($userid),
							bigintval($GLOBALS['refback_level']),
							bigintval($GLOBALS['refback_refid'][$GLOBALS['refback_level']])
						), __FUNCTION__, __LINE__);
				}

				// Do we have no entry?
				/* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'userid=' . $userid . ',level=' . $GLOBALS['refback_level'] . ',ref=' . $GLOBALS['refback_refid'][$GLOBALS['refback_level']] . ',minus=' . $minus . ',numRows=' . SQL_NUMROWS($result) . ' - FOUND!');
				if (SQL_HASZERONUMS($result)) {
					// Insert this level
					/* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'userid=' . $userid . ',level=' . $GLOBALS['refback_level'] . ',ref=' . $GLOBALS['refback_refid'][$GLOBALS['refback_level']] . ' - ADD!');
					SQL_QUERY_ESC("INSERT INTO `{?_MYSQL_PREFIX?}_user_refs` (`userid`, `level`, `refid`) VALUES (%s,%s,%s)",
						array(
							bigintval($userid),
							$GLOBALS['refback_level'],
							bigintval($GLOBALS['refback_refid'][$GLOBALS['refback_level']])
						), __FUNCTION__, __LINE__);

					// Move to next referal level and count his counter one up!
					$GLOBALS['refback_level']++;
					updateRefbackTable($GLOBALS['refback_refid'][($GLOBALS['refback_level'] - 1)]);
				} // END - if

				// Do we have another level here?
				if ((($GLOBALS['refback_level']-$minus) > 0) && ($old > 0)) {
					// Restore old one
					$userid = $old;

					// Shall we add this as well?
					$result = SQL_QUERY_ESC("SELECT `id` FROM `{?_MYSQL_PREFIX?}_user_refs` WHERE `userid`=%s AND `level`=%s AND `refid`=%s LIMIT 1",
						array(
							bigintval($userid),
							(bigintval($GLOBALS['refback_level']) - $minus),
							bigintval($GLOBALS['refback_refid'][$GLOBALS['refback_level']])
						), __FUNCTION__, __LINE__);

					// Do we have no entry?
					/* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'userid=' . $userid . ',level=' . ($GLOBALS['refback_level'] - $minus) . ',minus=' . $minus . ',ref=' . $GLOBALS['refback_refid'][$GLOBALS['refback_level']] . ',numRows=' . SQL_NUMROWS($result) . ' - BACK!');
					if (SQL_HASZERONUMS($result)) {
						// Insert this level
						SQL_QUERY_ESC("INSERT INTO `{?_MYSQL_PREFIX?}_user_refs` (`userid`, `level`, `refid`) VALUES (%s,%s,%s)",
							array(
								bigintval($userid),
								($GLOBALS['refback_level'] - $minus),
								bigintval($GLOBALS['refback_refid'][$GLOBALS['refback_level']])
							), __FUNCTION__, __LINE__);
						/* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'userid=' . $userid . ',level=' . ($GLOBALS['refback_level'] - $minus) . ',ref=' . $GLOBALS['refback_refid'][$GLOBALS['refback_level']] . ' - RETURNED!');
					} // END - if
				} // END - if
			} // END - if
		} // END - while

		// Free memory
		SQL_FREERESULT($result_refid);
	} // END - if

	// When he has a referal...
	if ((($GLOBALS['refback_refid'][$GLOBALS['refback_level']] == $userid) || ($GLOBALS['refback_refid'][$GLOBALS['refback_level']] == '0')) && (isExtensionInstalledAndNewer('cache', '0.1.2')) && (!isset($GLOBALS['refback_cached']))) {
		// Remove cache here
		/* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'userid=' . $userid . ',ref=' . $GLOBALS['refback_refid'][$GLOBALS['refback_level']] . ' - CACHE!');
		if ($GLOBALS['cache_instance']->loadCacheFile('refback')) $GLOBALS['cache_instance']->removeCacheFile();
		$GLOBALS['refback_cached'] = 1;
	} // END - if

	// "Walk" back here
	$GLOBALS['refback_level']--;
	/* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'userid=' . $userid . ',refback_level=' . $GLOBALS['refback_level'] .' - LEVEL!');

	// Fix empty refid
	if (!isset($GLOBALS['refback_refid'][$GLOBALS['refback_level']-1])) {
		$GLOBALS['refback_refid'][$GLOBALS['refback_level']-1] = '0';
	} // END - if
	/* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'userid=' . $userid . ',refback_level=' . $GLOBALS['refback_level'] . ',ref=' . $GLOBALS['refback_refid'][$GLOBALS['refback_level']-1] . ' - EXIT!');
}

// "Getter" for array for user refs in given level
function getArrayFromRefbackUserRefs ($userid, $level) {
	/* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'userid=' . $userid . ',level=' . $level . ' - ENTERED!');
	// Default is no refs and no nickname
	$refs = array();

	// Get refs from database
	$result = SQL_QUERY_ESC("SELECT
	r.`id`,
	r.`refid`,
	r.`refback`,
	r.`points`,
	d.`status`,
	d.`joined`,
	d.`refid` AS `level_refid`
FROM
	`{?_MYSQL_PREFIX?}_user_refs` AS r
LEFT JOIN
	`{?_MYSQL_PREFIX?}_user_data` AS d
ON
	r.`refid`=d.`userid`
WHERE
	r.`userid`=%s AND
	r.`level`=%s
ORDER BY
	r.`refid` ASC",
		array(bigintval($userid), bigintval($level)), __FUNCTION__, __LINE__);

	// Are there some entries?
	if (!SQL_HASZERONUMS($result)) {
		// Fetch all entries
		while ($row = SQL_FETCHARRAY($result)) {
			// Add row
			$refs[$row['id']] = $row;
		} // END - while
	} // END - if

	// Free result
	SQL_FREERESULT($result);

	// Return result
	/* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'userid=' . $userid . ',level=' . $level . ',refs()=' . count($refs) . ' - EXIT!');
	return $refs;
}

// "Getter" for refback entry
function getArrayFromUserRefbackData ($id) {
	/* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'id=' . $id . ' - ENTERED!');
	// Default data is nothing
	$data = array();

	// Secure id number
	$id = bigintval($id);

	// Get entry from database
	$result = SQL_QUERY_ESC("SELECT `id`, `userid`, `refid`, `refback`, `level`, `points` FROM `{?_MYSQL_PREFIX?}_user_refs` WHERE `id`=%s AND `userid`=%s LIMIT 1",
		array($id, getMemberId()), __FUNCTION__, __LINE__);

	// Is there an entry?
	if (SQL_NUMROWS($result) == 1) {
		// Fetch data
		$data = SQL_FETCHARRAY($result);
	} // END - if

	// Free result
	SQL_FREERESULT($result);

	// Return entry
	/* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'id=' . $id . ',data()=' . count($data) . ' - EXIT!');
	return $data;
}

// Update refback percents (but with some sanity-checks!
function updateMemberRefbackPercents ($id, $percents) {
	/* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'id=' . $id . ',percents=' . $percents . ' - ENTERED!');
	// Default status is failed
	$status['message'] = '{--MEMBER_REFBACK_ERROR_GENERAL--}';

	// Do some sanity-checks
	if (!isMember()) {
		// No member!
		$status['message'] = '{--MEMBER_REFBACK_ERROR_NO_MEMBER--}';
		return $status;
	} elseif (''.$id.'' != ''.($id + 0).'') {
		// No number!
		$status['message'] = '{--MEMBER_REFBACK_ERROR_INVALID_ID_NUMBER--}';
		return $status;
	} elseif (($percents < 0) || ($percents > 100)) {
		// Percentage is not valid!
		$status['message'] = '{--MEMBER_REFBACK_ERROR_INVALID_PERCENTAGE--}';
		return $status;
	}

	// Get ref entry
	$dummy = getArrayFromUserRefbackData($id);

	// Is this valid?
	if (count($dummy) == 0) {
		// id does not belong to user!
		$status['message'] = '{--MEMBER_REFBACK_ERROR_ID_MISMATCH--}';
		return $status;
	} // END - if

	// Revert german commata
	$percents = convertCommaToDot($percents);

	// Update entry
	SQL_QUERY_ESC("UPDATE `{?_MYSQL_PREFIX?}_user_refs` SET `refback`='%s' WHERE `id`=%s AND `userid`=%s LIMIT 1",
		array($percents, $id, getMemberId()), __FUNCTION__, __LINE__);

	// Entry updated?
	if (SQL_HASZEROAFFECTED()) {
		// Entry not updated!
		$status['message'] = '{--MEMBER_REFBACK_ERROR_NOT_UPDATED--}';
		return $status;
	} // END - if

	// Prepare email content
	$dummy['percents'] = $percents;

	// Load member email template
	$mail = loadEmailTemplate('member_refback', $dummy, $dummy['refid']);

	// Send email to user
	sendEmail($dummy['refid'], '{--MEMBER_REFBACK_SUBJECT--}', $mail);

	// Send admin notification
	sendAdminNotification('{--ADMIN_REFBACK_SUBJECT--}', 'admin_refback', $dummy, getMemberId());

	// All fine!
	$status['ok'] = true;

	// Return status array
	/* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'id=' . $id . ',percents=' . $percents . ' - EXIT!');
	return $status;
}

// Checks wether refback is enabled
function isRefbackEnabled () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = (getConfig('refback_enabled') == 'Y');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// [EOF]
?>
